dojo.provide("zero.form.widget.Form");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("zero.form.common.components.UidMixin");
dojo.require("zero.form.widget.FieldWrapperContainer");
dojo.require("zero.form.widget.FormController");

dojo.declare("zero.form.widget.Form",
             [dijit._Widget, dijit._Templated, zero.form.common.components.NlsMixin], {

		widgetsInTemplate: true,
		formModel:"",
		templatePath: dojo.moduleUrl("zero","form/widget/templates/form.html"),
		dataId:"",
		content: null,
		title: "",
		description: "",
		fields: null,
		controller: null,
		dataItem: null,
		submitLabel: "",
		clearLabel:"",
		dataGridId:"",
		formFields: "",
		
		postMixInProperties: function()
		{
			this.inherited("postMixInProperties",arguments);
			this.submitLabel = this.getString("SUBMIT");
			this.saveLabel = this.getString("SAVE");
			this.clearLabel = this.getString("CLEAR");
		},
		
		startup: function()
		{
			this.inherited("startup",arguments);
			if ("" == this.formModel) 
				return;
			this.formFields = {};
			
			var loadingFromServerErrorString = this.getString("LOADING_FORM_MODEL_FAILED",{formModelName:this.formModel});
			dojo.xhrGet({
				url:this.formModel,
				load:dojo.hitch(this, "setContent"),
				handleAs: 'json',
				error: function()
						 {
					     		if(zlog.isDebug()) console.debug(loadingFromServerErrorString);
						 }
				
			}); 
			
		},
		
		onGridSelection: function()
		{
			var dataGrid = dijit.byId(this.dataGridId);
			var rows = dataGrid.grid.selection.getSelected();
         //      return;
        //    } 
       //     else {
                var item = dataGrid.grid.model.getRow(rows[0]).__dojo_data_item;
                this.setValues(item);
       //     }
		},
		
		clearDataItem: function()
		{
			this.setDataItem(null);
		},
		
		setDataStore: function(/*zero.resource.DataStore */ dataStore)
		{
			if (this.controller)
			{
				this.controller.setStore(dataStore);
			}
			else
			{
				throw new Error("zero.form.widget.Form: the form doesn't have a model");
			}
		},
		
        setDataItem: function(dataItem)
        {
            this.dataItem = dataItem;
            this.btnLabel.innerHTML = (dataItem)?this.saveLabel:this.submitLabel;
			this.toggleMutabilityOfFields(dataItem);

        },

		toggleMutabilityOfFields: function(dataItem)
		{
			var field;
			for(var fieldId in this.formFields)
			{
				field = this.formFields[fieldId];
				if(field.contentJson && field.contentJson.immutable)
				{
					field.setDisabled(dataItem);
				}
			}
		},

		setValues: function(/*object*/ values)
		{
			var field;
			for(var fieldId in this.formFields)
			{
				field = this.formFields[fieldId];
				if(field.contentJson && field.contentJson.dataId)
				{
					field.setValue(values[field.contentJson.dataId]);	
				}
			}
                        this.setDataItem(values);
		},

		removeContent: function()
		{
			this.wrapperContainerNode.removeAll();
		},

		getContent: function()
		{
			var content = {};
			content.model = this.dataId;
			content.title = this.getTitle();
			content.description = this.getDescription();
			this.addFieldsToContent(content);
			return content;
		},
		
		addFieldsToContent: function(content)
		{
			content.fields = [];
			var field = {};
			var tmpField;
			var fields = this.getFields();
			for(var fieldId in fields)
			{
				field = fields[fieldId].getContent();
				content.fields.push(field);
			}
			
		},
		
		getFields: function()
		{
			return this.formFields;
		},
		
		setContent: function(jsonData)
		{
			this.content = jsonData;
			this.dataId = jsonData.model;
			if (this.dataId)
			{
				var controllerParams = {resourceCollection:this.dataId};
				if(this.dataGridId)
				{
					var dataGrid = dijit.byId(this.dataGridId);
					controllerParams.store = dataGrid.store;
					dojo.connect(dataGrid.grid,"onSelected",this,"onGridSelection");
					dojo.connect(dataGrid,"deleteRows",this,"clearDataItem");
					dojo.connect(dataGrid,"reload",this,"clearDataItem");
					
				}
				this.controller = new zero.form.widget.FormController(controllerParams);
			}
			this.setTitle(jsonData.title);
			this.setDescription(jsonData.description);
			this.addFields( jsonData.fields);
		},
		
		setTitle: function(newTitle)
		{
			this.title = newTitle;
			this.refreshTitle();
		},
		
		refreshTitle: function()
		{
			this.titleNode.innerHTML = this.title;
		},
		
		setDescription: function(newDescription)
		{
			this.description = newDescription;
			this.refreshDescription();
		},
		
		refreshDescription: function()
		{
			this.descriptionNode.innerHTML =this.description ;
		},
		
		getDescription: function()
		{
			return this.description;
		},
		
		getTitle: function()
		{
			return this.title;
		},
		
		addFields: function(fields)
		{
			var field;
			this.formFields = {};
			for(var i=0, l = fields.length;i<l;  i++)
			{
				this.addField(fields[i]);
			}			
		},
		
		addField: function(fieldJson)
		{
			var wrapper = this.wrapperContainerNode.createElement( 
				{ contentJson: fieldJson,
				  title: fieldJson.title,
				  hint: fieldJson.hint,
				  type: fieldJson.type,
				  containingForm:this,
				  blockEventsInitiation:this.blockEventsInitiation});
			this.formFields[fieldJson.shortName] = wrapper.getWidget();			
		},
	
		
		getValuesForDB: function()
		{
			var values = {};
			var field;
			for(var fieldId in this.formFields)
			{
				field = this.formFields[fieldId];
				if(field.contentJson && field.contentJson.dataId)
				{
					values[field.contentJson.dataId] = field.getValueForDB();	
				}
			}
			return values;
		},
		
		submitPressed: function()
		{
			this.doSubmit();
		},
		
		doSubmit: function()
		{
			if (this.controller)
			{
				if(this.dataItem)
				{
					this.controller.updateItem(this.dataItem,this.getValuesForDB());
				}
				else
				{
					this.setDataItem(this.controller.addItem(this.getValuesForDB()));
				}
			}
		},
		
		clearPressed: function()
		{
			var field;
			for(var fieldId in this.formFields)
			{
				this.formFields[fieldId].setValue("");
			}
			this.setDataItem(null);
		}
});